// Copyright © 2020 Amazon Web Services
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.notification.util;

import com.amazonaws.services.cognitoidp.AWSCognitoIdentityProvider;
import com.amazonaws.services.cognitoidp.AWSCognitoIdentityProviderClientBuilder;
import com.amazonaws.services.cognitoidp.model.InitiateAuthRequest;
import com.amazonaws.services.cognitoidp.model.InitiateAuthResult;

import org.opengroup.osdu.core.aws.iam.IAMConfig;

import java.util.HashMap;
import java.util.Map;

public class AwsCognitoClient {

    // Parameter value locations
    private final static String USERNAME_PARAM = "USERNAME";
    private final static String PASSWORD_PARAM = "PASSWORD";
    private final static String COGNITO_CLIENT_ID_PROPERTY = "AWS_COGNITO_CLIENT_ID";
    private final static String COGNITO_AUTH_FLOW_PROPERTY = "AWS_COGNITO_AUTH_FLOW";
    private final static String COGNITO_AUTH_PARAMS_USER_PROPERTY = "AWS_COGNITO_AUTH_PARAMS_USER";
    private final static String COGNITO_AUTH_PARAMS_PASSWORD_PROPERTY = "AWS_COGNITO_AUTH_PARAMS_PASSWORD";


    String awsCognitoClientId;
    String awsCognitoAuthFlow;
    String awsCognitoAuthParamsUser;
    String awsCognitoAuthParamsPassword;
    AWSCognitoIdentityProvider provider;

    public AwsCognitoClient() {
        this.awsCognitoClientId = AwsConfig.getAWSCognitoClientId();
        this.awsCognitoAuthFlow = AwsConfig.getAWSCognitoAuthFlow();
        this.awsCognitoAuthParamsUser = AwsConfig.getAWSCognitoUser();
        this.awsCognitoAuthParamsPassword = AwsConfig.getAWSCognitoPassword();
        this.provider =generateCognitoClient();
    }

    public AwsCognitoClient(String awsCognitoClientId, String awsCognitoAuthFlow, String awsCognitoAuthParamsUser
            , String awsCognitoAuthParamsPassword) {
        this.awsCognitoClientId = awsCognitoClientId;
        this.awsCognitoAuthFlow = awsCognitoAuthFlow;
        this.awsCognitoAuthParamsUser = awsCognitoAuthParamsUser;
        this.awsCognitoAuthParamsPassword = awsCognitoAuthParamsPassword;
       this.provider = generateCognitoClient();


    }

    public String getToken(String username, String password,String tokenType){
        Map<String, String> authParameters = new HashMap<>();
        authParameters.put(USERNAME_PARAM, username);
        authParameters.put(PASSWORD_PARAM, password);

        InitiateAuthRequest request = new InitiateAuthRequest();
        request.setClientId(awsCognitoClientId);
        request.setAuthFlow(awsCognitoAuthFlow);
        request.setAuthParameters(authParameters);
        String token="";
        InitiateAuthResult result = this.provider.initiateAuth(request);
        if(tokenType.equals("session"))
            token= result.getSession();
        else if(tokenType.equals("bearer"))
            token= "Bearer "+ result.getAuthenticationResult().getAccessToken();
        return token;
    }


    public AWSCognitoIdentityProvider getProvider() {
        return provider;
    }

    public  static AWSCognitoIdentityProvider generateCognitoClient()
    {
        String region = System.getenv("AWS_COGNITO_REGION");
        AWSCognitoIdentityProviderClientBuilder builder = AWSCognitoIdentityProviderClientBuilder.standard()
                .withCredentials(IAMConfig.amazonAWSCredentials());
        if ( region!= null) {
            builder.withRegion(region);
        }else{
            builder.withRegion(AwsConfig.getAwsRegion());
        }
        return  builder.build();
    }
}
